@@ -1,6 +1,6 @@
 /*
 	FATSort, utility for sorting FAT directory structures
-	Copyright (C) 2004 Boris Leidner <fatsort(at)formenos.de>
+	Copyright (C) 2018 Boris Leidner <fatsort(at)formenos.de>
 
 	This program is free software; you can redistribute it and/or
 	modify it under the terms of the GNU General Public License
@@ -29,9 +29,10 @@
 #include "errors.h"
 #include "stringlist.h"
 #include "regexlist.h"
+#include "deviceio.h"
 #include "mallocv.h"
 
-u_int32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE,
+uint32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE,
 	OPT_ORDER, OPT_LIST, OPT_REVERSE, OPT_FORCE, OPT_NATURAL_SORT,
 	OPT_RECURSIVE, OPT_RANDOM, OPT_MORE_INFO, OPT_MODIFICATION,
 	OPT_ASCII, OPT_REGEX;
@@ -63,8 +64,8 @@ int32_t addDirPathToStringList(struct sStringList *stringList, const char (*str)
 	len=strlen((char*)str);
 
 	// determine whether we have to add slashes
-	if (((const char*) str)[0] != '/') prefix=1;
-	if (((const char*) str)[len-1] != '/') suffix=1;
+	if (((const char*) str)[0] != DIRECTORY_SEPARATOR) prefix=1;
+	if (((const char*) str)[len-1] != DIRECTORY_SEPARATOR) suffix=1;
 
 	// allocate memory for string
 	newStr=malloc(prefix+len+suffix+1);
@@ -75,9 +76,9 @@ int32_t addDirPathToStringList(struct sStringList *stringList, const char (*str)
 
 	// copy string to new structure including missing slashes
 	newStr[0] = '\0';
-	strncat(newStr, "/", prefix);
+	if (prefix) newStr[0] = DIRECTORY_SEPARATOR;
 	strncat(newStr, (const char*) str, len);
-	strncat(newStr, "/", suffix);
+	if (suffix) newStr[prefix+len] = DIRECTORY_SEPARATOR; 
 
 	if (prefix+len+suffix > MAX_PATH_LEN) {
 		newStr[MAX_PATH_LEN] = '\0';
@@ -176,13 +177,23 @@ int32_t parse_options(int argc, char *argv[]) {
 	// sort by using locale collation order
 	OPT_ASCII = 0;
 
-	// default locale from environment
-	OPT_LOCALE = malloc(1);
+#ifdef __MINGW__
+#define WIN_LOCALE "C"
+	OPT_LOCALE = malloc(6);
 	if (OPT_LOCALE == NULL) {
 		stderror();
 		return -1;
 	}
-	OPT_LOCALE[0] = '\0';
+	strncpy(OPT_LOCALE, WIN_LOCALE, strlen(WIN_LOCALE));
+#else
+// default locale from environment für *nix
+OPT_LOCALE = malloc(1);
+if (OPT_LOCALE == NULL) {
+	stderror();
+	return -1;
+}
+OPT_LOCALE[0] = '\0';
+#endif
 
 	// empty string lists for inclusion and exclusion of dirs
 	if ((OPT_INCL_DIRS=newStringList()) == NULL) {
@@ -301,7 +312,7 @@ int32_t parse_options(int argc, char *argv[]) {
 			case 'q' : OPT_QUIET = 1; break;
 			case 'r' : OPT_REVERSE = -1; break;
 			case 'R' : OPT_RANDOM = 1; break;
-      case 't' : OPT_MODIFICATION = 1; break;
+			case 't' : OPT_MODIFICATION = 1; break;
 			case 'v' : OPT_VERSION = 1; break;
 			case 'L' :
 				OPT_LOCALE=realloc(OPT_LOCALE, strlen(optarg)+1);
@@ -336,4 +347,6 @@ void freeOptions() {
 	freeStringList(OPT_EXCL_DIRS_REC);
 	freeStringList(OPT_IGNORE_PREFIXES_LIST);
 	free(OPT_LOCALE);
+	freeRegExList(OPT_REGEX_INCL);
+	freeRegExList(OPT_REGEX_EXCL);
 }
