@@ -1,6 +1,6 @@
 /*
 	FATSort, utility for sorting FAT directory structures
-	Copyright (C) 2004 Boris Leidner <fatsort(at)formenos.de>
+	Copyright (C) 2018 Boris Leidner <fatsort(at)formenos.de>
 
 	This program is free software; you can redistribute it and/or
 	modify it under the terms of the GNU General Public License
@@ -24,13 +24,11 @@
 #ifndef __options_h__
 #define __options_h__
 
-#include <sys/types.h>
-#include "platform.h"
 #include "FAT_fs.h"
 #include "stringlist.h"
 #include "regexlist.h"
 
-extern u_int32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE,
+extern uint32_t OPT_VERSION, OPT_HELP, OPT_INFO, OPT_QUIET, OPT_IGNORE_CASE,
 		OPT_ORDER, OPT_LIST, OPT_REVERSE, OPT_FORCE, OPT_NATURAL_SORT,
 		OPT_RECURSIVE, OPT_RANDOM, OPT_MORE_INFO, OPT_MODIFICATION,
 		OPT_ASCII, OPT_REGEX;
